#include <string.h>
#include "tsys.h"
#include "tobj.h"
/* for some defines */
#include "tfamily.h"

#include "prvclass/tfamily.h"

void tfamily_addchild(struct tobj *father, struct tobj *child)
{
	struct tfamily *family;
	family = father->data;

	if (family->first) {
		struct tobj *obj2;
		/* finding last one */
		for (obj2 = family->first;
			obj2->next != NULL; obj2 = obj2->next);

		obj2->next = child;
	} else {
		family->first = child;
	}

	if (!family->active)
		tfamily_activatechild(father, child);


	child->next = NULL;
	child->parent = father;

}
void tfamily_delchild(struct tobj *father, struct tobj *child)
{
	struct tfamily *family;
	family = father->data;

	if (family->first == child) {
		if (family->active == child)
			tobj_activate_next2(father);
		family->first = child->next;
	} else {
		struct tobj *obj2;
		/* finding prev one */
		for (obj2 = family->first;
		     obj2->next != child && obj2 != NULL; obj2 = obj2->next);

		if (family->active == child)
			tobj_activate_next2(father);
		obj2->next = child->next;
	}
}
/* if object is active like its father */
int tfamily_isactive(struct tobj *father, struct tobj *child)
{
	struct tfamily *family;
	family = father->data;

	if (family->active == child && tobj_isactive(father))
		return 1;
	else
		return 0;
}
/* if object is active one in given father.
 * warning: it isn't eqal to above function
 */
int tfamily_isactivechild(struct tobj *father, struct tobj *child)
{
	struct tfamily *family;
	family = father->data;

	if (family->active == child )
		return 1;
	else
		return 0;
}
struct tobj *tfamily_firstchild(struct tobj *father)
{
	struct tfamily *family;
	family = father->data;

	return family->first;
}
struct tobj *tfamily_activechild(struct tobj *father)
{
	struct tfamily *family;
	family = father->data;

	return family->active;
}
void tfamily_activatechild(struct tobj *father, struct tobj *child)
{
	struct tfamily *family;
	family = father->data;

	if (child && child->activeable && !child->hidden && !child->garbage) {
		family->active = child;
		tsystem->refresh = 1;
	} else if ( !child ) {
		family->active = NULL;
		tsystem->refresh = 1;
	}
}
void tfamily_activatenextchild(struct tobj *father)
{
	struct tfamily *family;
	struct tobj *obj;
	family = father->data;


	if (family->active) {
		obj = family->active;

		/* change child (repeat it until it is activeable) */
		do {
			obj = obj->next;
			if (!obj)
				obj = family->first;

			/* We don't wonna to do forever-loop.
			   Active object will stay active. */
			if (obj == family->active)
				break;

		} while (!obj->activeable || obj->hidden || obj->garbage);

		family->active = obj;
	} else if (family->first) {
		obj = family->first;

		/* We do loop if first one isn't activeable */
		while (!obj->activeable || obj->hidden || obj->garbage) {
			obj = obj->next;

			/* End of child-list.
			   No object will be active */
			if (!obj)
				break;
		}
		family->active = obj;
	}


	tsystem->refresh = 1;
}

void tfamily_activateprevchild(struct tobj *father)
{
	struct tfamily *family;
	struct tobj *obj, *last = NULL;
	family = father->data;


	if (family->active) {
		obj = family->active;

		do {
			if (obj->activeable && !obj->hidden && !obj->garbage)
				last = obj;
			
			obj = obj->next;
			if (!obj)
				obj = family->first;

		} while (obj != family->active);

		if (last)
			family->active = last;
		
	} else if (family->first) {
		obj = family->first;

		/* We do loop if first one isn't activeable */
		while ( obj ) {
			obj = obj->next;

			if (obj->activeable && !obj->hidden && !obj->garbage)
				last = obj;
		}
		if (last)
			family->active = last;
	}


	tsystem->refresh = 1;
}
int tfamily_doevent(struct tobj *obj, int keycode)
{
	struct tfamily *family;
	family = (struct tfamily *) obj->data;

	if (family->active && tobj_doevent(family->active, keycode) )
		return 1;
	else 
		return 0;
}

void tfamily_dorefresh(struct tobj *obj)
{
	struct tobj *tmpobj;
	struct tfamily *family;
	family = (struct tfamily *) obj->data;
	
	/* redrawing all objects */
	if (family->first)
		for (tmpobj = family->first; tmpobj != NULL;
					tmpobj = tmpobj->next)
			if (tmpobj != family->active)
				tobj_dorefresh(tmpobj);

	/* and last one - active */
	if (family->active)
		tobj_dorefresh(family->active);

}

void tfamily_cleanup(struct tobj *obj)
{
	struct tfamily *family;
	family = obj->data;

	/* Freeing all objects */
	if (family->first) {
		struct tobj *o;
		for ( o = family->first; o ; o=o->next)
			tobj_free(o);
	}

}
void tfamily_garbage(struct tobj *obj)
{
	struct tfamily *family;
	family = obj->data;

	if (family->first) {
		struct tobj *obj, *next;
		for ( obj = family->first; obj ; obj=next){
			next = obj->next;
			tobj_garbage(obj);
		}
	}

	if ( obj->garbage ){
		free(family);
		obj->data=NULL;
	}
}

int tfamily_setup(struct tobj *obj, struct tobj_fnc *fnc)
{
	struct tfamily *family;
	family = (struct tfamily*)obj->data;

	family->first = NULL;
	family->active = NULL;

	fnc->free = NULL;
	fnc->garbage = tfamily_garbage;
	fnc->dorefresh = tfamily_dorefresh;
	fnc->doevent = tfamily_doevent;
	fnc->domethod = NULL;

	return 1;
}
