#include "tsys.h"
#include "tobj.h"
#include "twin.h"
/* defines of events and attrs */
#include "tlabel.h"
#include <string.h>

#define LEFT	(0L)
#define RIGHT	(1L)
#define CENTER	(2L)

struct tlabel {
	char *label;
	int align;
};


static int linelen(char *s)
{
	char *s0;
	s0 = s;
	while (*s && *s != '\n')
		s++;

	return (int) (s - s0);
}
static char *linenext(char *s)
{
	while (*s) {
		if (*s == '\n') {
			s++;
			break;
		}
		s++;
	}

	return s;
}
void tlabel_dorefresh(struct tobj *obj)
{
	unsigned int bx, by;
	unsigned int x, len, i, j;
	struct tlabel *label;
	char *s;

	bx = t_wherex();
	by = t_wherey();

	label = (struct tlabel *) obj->data;

	t_textcolor(obj->fgcolor);
	t_textbackground(obj->bgcolor);

	s = label->label;
	for (i = 0; i < obj->h; i++) {
		len = linelen(s);
		if (label->align == CENTER)
			x = obj->w / 2 - len / 2;
		else if (label->align == RIGHT)
			x = obj->w - len;
		else
			x = 0;

		t_gotoxy(bx, by + i);
		for (j = 0; j < obj->w; j++)
			if (j >= x && j < len + x)
				t_putchar(s[j - x]);
			else
				t_putchar(' ');

		s = linenext(s);
	}
}
static int tlabel_set(struct tobj *obj, struct tobj_set *tos)
{
	struct tlabel *label;
	label = obj->data;

	switch (tos->attr) {
	case TLABEL_LABEL:
		if (label->label)
			free(label->label);
		if (!(label->label = strdup((char *) tos->data)))
			tsys_error_exit
			    ("ERROR: While changing labels in tlabel.\n");
		tobj_dorefresh(obj);
		break;
	case TLABEL_ALIGN:
		label->align = (int) (tos->data);
		tobj_dorefresh(obj);
		break;
	default:
		return 0;
	}
	return 1;
}

static int tlabel_get(struct tobj *obj, struct tobj_get *tog)
{
	struct tlabel *label;
	label = obj->data;

	switch (tog->attr) {
	case TLABEL_LABEL:
		*(tog->data) = TPDATA(label->label);
		break;
	case TLABEL_ALIGN:
		*(tog->data) = label->align;
		break;
	default:
		return 0;
	}
	return 1;
}
int tlabel_domethod(struct tobj *obj, long method, long data)
{
	switch (method) {
	case TOBJ_SET:
		return tlabel_set(obj, (struct tobj_set *) data);
	case TOBJ_GET:
		return tlabel_get(obj, (struct tobj_get *) data);
	default:
		return 0;
	}
}
void tlabel_cleanup(struct tobj *obj)
{
	struct tlabel *label;
	label = (struct tlabel *) obj->data;

	if (label->label){
		free(label->label);
		label->label = NULL;
	}
}
void tlabel_free(struct tobj *obj)
{
	struct tlabel *label;
	label = (struct tlabel *) obj->data;

	tlabel_cleanup( obj );
	
	if (label)
		free(label);
}

struct tobj *tlinelabel_init(struct tobj *parent, int x, int y, char *text)
{
	return tlabel_init(parent, x, y, strlen(text), 1, text, LEFT);
}

int tlabel_setup(struct tobj *obj, struct tobj_fnc *fnc,
			 char *text, int align)
{
	struct tlabel *label;
	char nomem[] = "ERROR: While allocating memory for tlabel.\n";
	label = (struct tlabel *) obj->data;

	if (!(label->label = strdup(text))) {
		tsys_error(nomem);
		return 0;
	} else {

		label->align = align;
		
		fnc->free = &tlabel_free;
		fnc->dorefresh = &tlabel_dorefresh;
		fnc->domethod = &tlabel_domethod;

		return 1;
	}
}

struct tobj *tlabel_init(struct tobj *parent, int x, int y, int w, int h,
			 char *text, int align)
{
	struct tobj *obj;
	struct tlabel *label;
	char nomem[] = "ERROR: While allocating memory for tlabel.\n";

	if (!(obj = calloc(1, sizeof(struct tobj)))) {
		tsys_error(nomem);
		return 0;
	}
	if (!(label = calloc(1, sizeof(struct tlabel)))) {
		tsys_error(nomem);
		return 0;
	} else {
		obj->data = label;

		obj->x = x;
		obj->y = y;
		obj->h = h;
		obj->w = w;

		/* setting default values */
                obj->fgcolor = TLABELFG;
		obj->bgcolor = (char) txget(parent, TOBJ_BGCOLOR);
		obj->activeable = 0;
		obj->parent = parent;

		if (( tlabel_setup( obj, &obj->fnc , text, align) )){
			tobj_init(parent, obj);
			return obj;
		} else
			return 0;
		
	}
}
