#include "tsys.h"
#include "tobj.h"
#include "twin.h"
/* defines of events and methods */
#include "tscroll.h"
#include "miscfunc.h"
#include <stdio.h>
#include <math.h>


#include "prvclass/tscroll.h"

void tscroll_dorefresh(struct tobj *obj)
{
	int bx, by;
	struct tscroll *scroll;
	int objactive = 0;
	unsigned long dpos, dlen, slen, i;

	scroll = (struct tscroll *) obj->data;

	bx = t_wherex();
	by = t_wherey();

	if (tobj_isactive(obj))
		objactive = 1;


	if (scroll->vert)
		slen = obj->h;
	else
		slen = obj->w;


	if (slen < scroll->lenght) {
		/* calculating lenght of drag bar (proportion) */
		dlen = ceil(((float) slen / scroll->lenght) * slen);

		/* calculating position of drag bar (proportion)
		 * (in polish)
		 * (scroll->firstline + 1) - numer pozycji dragbara (zaczynajac
		 *    od 1)
		 * (obj->h - len + 1) - ilosc mozliwych pozycji jakie
		 *    moze przyjac dragbar
		 * (scroll->nlines - obj->h + 1) - ilosc mozliwych pozycji
		 *    jakie moze przyjac drag podczas skrolowania
		 *    pionowego/poziomowego
		 */
		dpos = ceil((float) (scroll->pos + 1)
			    * (slen - dlen + 1) / (scroll->lenght - slen +
						   1)) - 1;
	} else {
		dpos = 0;
		dlen = slen;
	}

	/* clearing */
	t_textbackground(obj->bgcolor);
	t_textcolor(obj->bgcolor2);
	if (scroll->vert)
		for (i = 0; i < slen; i++) {
			t_gotoxy(bx, by + i);
			t_putchar(obj->bgchar);
	} else {
		t_gotoxy(bx, by);
		for (i = 0; i < slen; i++)
			t_putchar(obj->bgchar);
	}


	/* drawing drag-bar */
	if (objactive)
		t_textbackground(obj->fgcolor);
	else
		t_textbackground(obj->fgcolor2);

	if (scroll->vert)
		for (i = dpos; i < dlen + dpos; i++) {
			t_gotoxy(bx, by + i);
			t_putchar(' ');
	} else {
		t_gotoxy(bx + dpos, by);
		for (i = 0; i < dlen; i++)
			t_putchar(' ');
	}

}
static int tscroll_set(struct tobj *obj, struct tobj_set *tos)
{
	struct tscroll *scroll;
	scroll = obj->data;

	switch (tos->attr) {
	case TSCROLL_POS:
		scroll->pos = tos->data;
		tobj_dorefresh(obj);
		break;
	case TSCROLL_LENGHT:
		scroll->lenght = tos->data;
		tobj_dorefresh(obj);
		break;
	default:
		return 0;
	}

	return 1;
}

static int tscroll_get(struct tobj *obj, struct tobj_get *tog)
{
	struct tscroll *scroll;
	scroll = (struct tscroll*)obj->data;

	switch (tog->attr) {
	case TSCROLL_POS:
		*(tog->data) = scroll->pos;
		break;
	case TSCROLL_LENGHT:
		*(tog->data) = scroll->lenght;
		break;
	default:
		return 0;
	}

	return 1;
}
int tscroll_domethod(struct tobj *obj, long method, long data)
{
	switch (method) {
	case TOBJ_SET:
		return tscroll_set(obj, (struct tobj_set *) data);
	case TOBJ_GET:
		return tscroll_get(obj, (struct tobj_get *) data);
	}
	return 0;
}

void tscroll_cleanup(struct tobj *obj)
{
	/* do nothing */
	(void)obj;
}

void tscroll_free(struct tobj *obj)
{
	tscroll_cleanup( obj );

	if (obj->data)
		free(obj->data);

}

int tscroll_setup(struct tobj *obj, struct tobj_fnc *fnc, int vert)
{
	struct tscroll *scroll;
	scroll = (struct tscroll*)obj->data;

	
	scroll->vert = vert;
	scroll->pos = 0;
	scroll->lenght = 1;

	fnc->free = &tscroll_free;
	fnc->dorefresh = &tscroll_dorefresh;
	fnc->domethod = &tscroll_domethod;
	fnc->doevent = NULL;

	return 1;
}
struct tobj *tscroll_init(struct tobj *parent, int x, int y, int len, int vert)
{
	struct tobj *obj;
	struct tscroll *scroll;
	char nomem[] = "ERROR: While allocating memory for tscroll.\n";

	if (!(obj = calloc(1, sizeof(struct tobj)))) {
		tsys_error(nomem);
		return 0;
	}
	if (!(scroll = calloc(1, sizeof(struct tscroll)))) {
		tsys_error(nomem);
		return 0;
	} else {
		obj->data = scroll;

		obj->x = x;
		obj->y = y;
		if (vert) {
			obj->h = len;
			obj->w = 1;
		} else {
			obj->w = len;
			obj->h = 1;
		}

		/* setting default values */
                obj->fgcolor = TSCROLLFG;
                obj->fgcolor2 = TSCROLLFG2;
                obj->bgcolor = TSCROLLBG;
                obj->bgcolor2 =TSCROLLBG2;
		obj->activeable = 0;
		obj->parent = parent;

		obj->bgchar = t_gethatch((int) txget(parent, TWIN_FRAME));

		if (( tscroll_setup( obj , &obj->fnc , vert ) )){
			tobj_init(parent, obj);
			return obj;
		}
		else
			return 0;
	}
}
