#include <string.h>
#include "tsys.h"
#include "tobj.h"
/* for some defines */
#include "twin.h"

#include "prvclass/twin.h"

static int twin_set(struct tobj *obj, struct tobj_set *tos)
{
	struct twin *win;
	win = obj->data;
	
	switch (tos->attr) {
	case TWIN_TITLE:
		if (!( win->title = strdup( (char*) tos->data ) ))
			tsys_error_exit("ERROR: While setting window title.\n");
		tobj_dorefresh( obj );
		break;
	case TWIN_FRAME:
		win->frame = tos->data;
		break;
	case TWIN_MOVEABLE:
		win->moveable = tos->data;
		break;
	case TWIN_SIZEABLE:
		win->sizeable = tos->data;
		break;
	default:
		if ( win->family_fnc.domethod
			&& (*win->family_fnc.domethod)( obj,
					TOBJ_SET, TPDATA(tos)) )
			break;
		else
			return 0;
	}
	return 1;
}

static int twin_get(struct tobj *obj, struct tobj_get *tog)
{
	struct twin *win;
	win = obj->data;

	switch (tog->attr) {
	case TWIN_TITLE:
		*(tog->data) = TPDATA( win->title );
		break;
	case TWIN_FRAME:
		*(tog->data) = win->frame;
		break;
	case TWIN_MOVEABLE:
		*(tog->data) = win->moveable;
		break;
	case TWIN_SIZEABLE:
		*(tog->data) = win->sizeable;
		break;
	default:
		if ( win->family_fnc.domethod
			&& (*win->family_fnc.domethod)( obj,
					TOBJ_GET, TPDATA(tog)) )
			break;
		else
			return 0;
	}
	return 1;
}
int twin_domethod(struct tobj *obj, long method, long data)
{
	struct twin *win;
	win = (struct twin *) obj->data;
	
	switch (method) {
	case TOBJ_SET:
		return twin_set(obj, (struct tobj_set *) data);
	case TOBJ_GET:
		return twin_get(obj, (struct tobj_get *) data);
	case TWIN_ACTIVATE_NEXT:
		tobj_activate_next(obj);
		break;
	case TWIN_OPEN:
		tset( obj , TOBJ_HIDDEN , 0 );
		break;
	case TWIN_CLOSE:
		tset( obj , TOBJ_HIDDEN , 1 );
		break;
	case TWIN_ACTIVATE_CHILD:
		// someone (read misto) should change it :)
		while (!tfamily_isactivechild(obj, (struct tobj *)data))
			tobj_activate_next(obj);
		break;
			
	default:
		if ( win->family_fnc.domethod
			&& (*win->family_fnc.domethod)( obj, method, data) )
			break;
		else
			return 0;
	}
	return 1;
}
static int twin_dochangemode(struct tobj *obj, int keycode)
{
	struct twin *win;
	win = (struct twin *) obj->data;

	switch (keycode) {
	case K_LEFT:
		if (win->moveable) {
			if (tobj_getx(obj) > 1)
				obj->x--;
			tsystem->refresh = 1;
			return 1;
		}
		break;
	case K_RIGHT:
		if (win->moveable) {
			if ( tobj_getx(obj) + obj->w - 1< t_screencols() )
				obj->x++;
			tsystem->refresh = 1;
			return 1;
		}
		break;
	case K_UP:
		if (win->moveable) {
			if (tobj_gety(obj) > 1)
				obj->y--;
			tsystem->refresh = 1;
			return 1;
		}
		break;
	case K_DOWN:
		if (win->moveable) {
			if ( tobj_gety(obj) + obj->h - 1< t_screenrows() )
				obj->y++;
			//obj->y = MIN(obj->parent->h - obj->h, obj->y + 1);
			tsystem->refresh = 1;
			return 1;
		}
		break;
	case K_RET:
	case K_ESC:
		win->changemode = 0;
		tobj_dorefresh(obj);
		return 1;
	}
	return 0;
}

/* Reacting on event or sending it to active object */
int twin_doevent(struct tobj *obj, int keycode)
{
	struct twin *win;
	win = (struct twin *) obj->data;

	if (win->changemode) {
		return twin_dochangemode(obj, keycode);
	} else if ( win->family_fnc.doevent
			&& (*win->family_fnc.doevent)(obj, keycode) )
		return 1;
	else {
		/* do something with event */
		switch (keycode) {
		case K_ALT_C:
		case K_A_F3:
			tevent_send(obj, EVENT_TYPE_USER, EVENT_TWIN_CLOSE);
			break;
		case K_ALT_M:
		case K_C_F5:
			if (win->moveable){
				win->changemode = 1;
				tobj_dorefresh(obj);
			}
			break;
		case K_TAB:
			if (obj == tsystem->rootwin) {
				twin_domethod(obj, TWIN_ACTIVATE_NEXT, 0);
				break;
			} else
				return 0;
		case K_ALT_N:
			twin_domethod(obj, TWIN_ACTIVATE_NEXT, 0);
			break;
		default:
			/* or return 0, if keycode wasn't used */
			return 0;
		}
	}
	return 1;
}

/* Redrawing whole window */
void twin_dorefresh(struct tobj *obj)
{
	unsigned int i, j;
	struct twin *win;
	unsigned int bx, by;

	win = (struct twin *) obj->data;
	bx = t_wherex();
	by = t_wherey();


	/* redraw window */
	t_textbackground(obj->bgcolor);
	if (tobj_isactive(obj))
		t_textcolor(obj->fgcolor);
	else
		t_textcolor(obj->fgcolor2);

	/* Cleaning window area */
	for (i = 0; i < obj->h; i++) {
		t_gotoxy(bx, by + i);
		for (j = 0; j < obj->w; j++)
			t_putchar(' ');
	}

	/* drawing frame */
	if (win->frame) {
		t_gotoxy(bx, by);
		t_drawframe(obj->w, obj->h, win->frame);
	}

	/* drawing title */
	if (win->title) {
		if (!win->changemode) {
			int x;
			x = (bx + (bx + obj->w)) / 2 -
			    ((strlen(win->title) + 2) / 2);
			t_gotoxy(x, by);
			t_putchar(' ');
			t_cputs(win->title);
			t_putchar(' ');
		}
	}

	if ( win->family_fnc.dorefresh ) 
		(*win->family_fnc.dorefresh )( obj );
			
}

void twin_cleanup(struct tobj *obj)
{
	struct twin *win;
	win = obj->data;

	tfamily_cleanup( obj );

	if (win->title){
		free(win->title);
		win->title=NULL;
	}

}
void twin_free(struct tobj *obj)
{
	struct twin *win;
	win = (struct twin*)obj->data;

	twin_cleanup( obj );

}


int twin_setup(struct tobj *obj, struct tobj_fnc *fnc, char *title)
{
	struct twin *win;
	char nomem[] = "ERROR: While allocating memory for twin.\n";
	win = (struct twin*)obj->data;

	win->frame = TWIN_FRAME_SINGLE;
	win->moveable = 1;
	win->sizeable = 0;
	
	if (title) {
		if (!(win->title = strdup(title))) {
			tsys_error(nomem);
			return 0;
		}
	} else
		win->title = NULL;

	if (!( tfamily_setup( obj , &win->family_fnc ) ))
		return 0;

	fnc->free = twin_free;
	fnc->garbage = win->family_fnc.garbage;
	fnc->dorefresh = twin_dorefresh;
	fnc->doevent = twin_doevent;
	fnc->domethod = twin_domethod;

	return 1;
}
struct tobj *twin_init(struct tobj *parent, int x, int y,
		       int width, int height, char *title)
{
	struct tobj *obj;
	struct twin *win;
	char nomem[] = "ERROR: While allocating memory for twin.\n";

	if (!(obj = calloc(1, sizeof(struct tobj)))) {
		tsys_error(nomem);
		return 0;
	}
	if (!(win = calloc(1, sizeof(struct twin)))) {
		tsys_error(nomem);
		return 0;
	} else {
		obj->data = win;

		obj->x = x;
		obj->y = y;
		obj->h = height;
		obj->w = width;

                obj->fgcolor = TWINFG;
                obj->fgcolor2 = TWINFG2;
                obj->bgcolor = TWINBG;
                obj->bgcolor2 =TWINBG2;

		obj->activeable = 1;
		obj->parent = parent;

		if (( twin_setup( obj , &obj->fnc , title ) )){
			tobj_init(parent, obj);
			return obj;
		} else
			return 0;
		
	}
}
